.toggle-switch {
  width: 32px;
}

#dashboard-container .mantine-Switch-track {
  background-color: var(--fill-subtle);
  border: 1px solid var(--border-enabled);
  border-radius: 16px;
  height: 16px;
  min-width: 32px;
  width: 32px;
}

#dashboard-container .mantine-Switch-track:focus {
  border: 2px solid var(--focus-focus);
}

#dashboard-container .mantine-Switch-input {
  margin: 0;
}

#dashboard-container .mantine-Switch-trackLabel {
  height: 16px;
  margin: 0;
  width: 32px;
}

#dashboard-container .mantine-Switch-thumb {
  background-color: var(--fill-medium-emphasis);
  border: none;
  height: 10px;
  width: 10px;
}

#dashboard-container input:checked + * > .mantine-11dx59s {
  background: var(--text-contrast-primary);
  left: calc(100% - 12px);
}

#dashboard-container input:checked + * > .mantine-69c9zd {
  background: var(--text-primary);
  border-color: var(--border-enabled);
}
