# TODO: eventually deduplicate page generation into a function rather than copying and pasting across files?
# TODO: think about the best way to do code examples, e.g.
# - do we want full dashboard example or plot-only example?
# - or both? Could be done using a toggle switch or multiple tabs.
# - a link to py.cafe showing the dashboard code?
