::-webkit-scrollbar {
  width: 12px;
}

::-webkit-scrollbar-thumb {
  background: var(--fill-subtle);
  border: 4px solid;
  border-color: var(--field-enabled);
}

::-webkit-scrollbar-thumb:hover {
  background: var(--fill-medium-emphasis);
}

#left-main::-webkit-scrollbar-thumb {
  border-color: var(--surfaces-bg-02);
}

.card::-webkit-scrollbar-thumb,
.card-nav::-webkit-scrollbar-thumb,
.hljs::-webkit-scrollbar-thumb {
  border-color: var(--surfaces-bg-card);
}

#page-components::-webkit-scrollbar-thumb {
  border-color: var(--main-container-bg-color);
}

.table-container::-webkit-scrollbar-thumb {
  border-color: var(--main-container-bg-color);
}

/* Can't control thumb color so hover is turned off */
.ag-root ::-webkit-scrollbar {
  width: 4px;
}

.ag-root ::-webkit-scrollbar-thumb:hover {
  background: var(--field-enabled);
}
